"""Unit tests configuration file."""

import logging

import pytest

import log


def pytest_configure(config):
    """Disable verbose output when running tests."""
    logging.basicConfig(level=logging.DEBUG)

    terminal = config.pluginmanager.getplugin('terminal')
    base = terminal.TerminalReporter

    class QuietReporter(base):
        """Reporter that only shows dots when running tests."""

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.verbosity = 0
            self.showlongtestinfo = False
            self.showfspath = False

    terminal.TerminalReporter = QuietReporter


@pytest.fixture(autouse=True)
def reset_state():
    log.state.default_level = None



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import logging

from log import logger


def describe_log():

    def it_sets_level_and_message(expect, caplog):
        logger.log(logging.DEBUG, "foobar")
        expect(caplog.records[-1].levelname) == 'DEBUG'
        expect(caplog.records[-1].message) == "foobar"



# pylint: disable=unused-variable,expression-not-assigned

from unittest.mock import call, patch

from log import helpers


def describe_init():

    @patch('logging.basicConfig')
    def with_verbosity_0(config, expect):
        helpers.init(format='%(message)s', verbosity=0)
        expect(config.mock_calls) == [call(format='%(message)s', level=40)]

    @patch('logging.basicConfig')
    def with_verbosity_1(config, expect):
        helpers.init(format='%(message)s', verbosity=1)
        expect(config.mock_calls) == [call(format='%(message)s', level=30)]

    @patch('logging.basicConfig')
    def with_verbosity_2(config, expect):
        helpers.init(format='%(message)s', verbosity=2)
        expect(config.mock_calls) == [call(format='%(message)s', level=20)]

    @patch('logging.basicConfig')
    def with_verbosity_3(config, expect):
        helpers.init(format='%(message)s', verbosity=3)
        expect(config.mock_calls) == [call(format='%(message)s', level=10)]

    @patch('logging.basicConfig')
    def with_verbosity_0_and_debug(config, expect):
        helpers.init(format='%(message)s', verbosity=0, debug=True)
        expect(config.mock_calls) == [call(format='%(message)s', level=10)]



"""Wrappers to eliminate boilerplate `logging` activities."""

import logging

from . import state
from .filters import relpath_format_filter

VERBOSITY_TO_LEVEL = {
    0: logging.ERROR,
    1: logging.WARNING,
    2: logging.INFO,
    3: logging.DEBUG,
}


def init(*, reset=False, debug=False, verbosity=None, **kwargs):
    if reset:
        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)

    custom_format = kwargs.get('format')
    if debug:
        state.default_level = logging.DEBUG
    elif verbosity is not None:
        state.default_level = VERBOSITY_TO_LEVEL[verbosity]

    kwargs['level'] = kwargs.get('level', state.default_level)
    kwargs['format'] = kwargs.get('format', state.default_format)
    logging.basicConfig(**kwargs)

    if custom_format:
        formatter = logging.Formatter(
            fmt=custom_format,
            datefmt=kwargs.get('datefmt'),
            style=kwargs.get('style', '%'),
        )
        for handler in logging.root.handlers:
            handler.setFormatter(formatter)

    install_additional_formats(logging.root)

    state.initialized = True


def install_additional_formats(logger):
    for handler in logger.handlers:
        handler.addFilter(relpath_format_filter)


def silence(*names, allow_info=False, allow_warning=False, allow_error=False):
    if allow_info:
        level = logging.INFO
    elif allow_warning:
        level = logging.WARNING
    elif allow_error:
        level = logging.ERROR
    else:
        level = logging.CRITICAL

    for name in names:
        logging.getLogger(name).setLevel(level)
